/**
 * JSTagsTagSupport.java $Revision: 207 $ $Date: 2007-10-12 15:08:01 +0200 (Fri, 12 Oct 2007) $
 *
 * Copyright 2007 jstags
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sourceforge.jstags.tags;

import java.io.IOException;
import java.util.Map;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

import net.sourceforge.jstags.exceptions.TemplateException;
import net.sourceforge.jstags.utils.template.TemplateUtils;

import org.apache.log4j.Logger;


/**
 * 
 * It's a base class for defining the tag handlers of tags
 * with no body 
 * 
 * Every tag extending this class will have the following components:
 * <ul>
 * <li>Implementation class: it is in charge of gathering all the 
 * information to create the javascript code used by the tag
 * </li>
 * <li>Template: it is a javascript file with the code used by a tag. It 
 * contains a set of parameters(i.e. effect options, target element, ...) 
 * that can be replaced with the particular values used by a tag invocation
 * </li>
 * <li>Parameters: set of pairs [name, value] with the values of every 
 * parameter included in a template. This pairs will be used in order
 * to replace the variables included in a template and get the javascript
 * to include in a jsp at translation time
 * </li>
 * </ul> 
 * 
 * @author sschz@users.sourceforge.net
 * @since 1.0
 * 
 */
public abstract class JSTagsTagSupport extends TagSupport {
	/**
	 * Logger for this class
	 */
	private static final Logger logger = Logger
			.getLogger(JSTagsTagSupport.class);

	private static final long serialVersionUID = -3924134604869121719L;

	/* (non-Javadoc)
	 * @see javax.servlet.jsp.tagext.TagSupport#doStartTag()
	 */
	public final int doStartTag() throws JspException {

		try {
			pageContext.getOut().write(getOutput());
		} catch (IOException e) {
			logger.error("There has been an error trying to get the effect declaration", e);
			throw new JspException("Error ", e);
		}
		
		return SKIP_BODY;
	}

	/**
	 * The tag code to be inserted in the jsp page. Such code will be retrieved by replacing, in the
	 * correct template, the previously set parameters
	 * 
	 * @return the JavaScript code that will perform the tag functionality
	 * @throws JspException
	 */
	protected final String getOutput()
		throws JspException {

		String output = new String();

		try {
			output = TemplateUtils.replaceValuesInTemplate(
					TemplateUtils.getTemplateAsString(getTemplateName()), getTemplateNameValuePairs());
		} catch (TemplateException e) {
			logger.error("There has been an error trying to replace variable names in the template"
					+ " named " + getTemplateName(), e);
			throw new JspException("Error trying to replace parameter values in the template", e);
		} 

		return output;		
	}

	/**
	 * @return name of the .js template used to create the JavaScript code used by this tag 
	 * @throws JspException
	 */
	protected abstract String getTemplateName() throws JspException;
	
	/**
	 * @return {@link Map} of [name, value] pairs representing all the name-value pair
	 * used to create the JavaScript code. Such pairs will be used to replace data inside the .js
	 * template in order to get the code of a certain effect.
	 */
	protected abstract Map<String, String> getTemplateNameValuePairs();

}
