/**
 * Effect.java $Revision: 417 $ $Date: 2007-10-31 00:50:11 +0100 (Wed, 31 Oct 2007) $
 *
 * Copyright 2007 jstags
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sourceforge.jstags.tags.effects;

import java.util.HashMap;
import java.util.Map;

import net.sf.json.JSONFunction;
import net.sf.json.JSONObject;
import net.sourceforge.jstags.naming.TemplateConstants;
import net.sourceforge.jstags.tags.JSTagsTagSupport;

import org.apache.commons.lang.StringUtils;
 
/**
 * It contains all the common details on every effect
 * 
 * All of the effects extending this class, depend on the
 * JavaScript library <a href="http://mochikit.com">Mochikit</a> 
 * 
 * @author sschz@users.sourceforge.net
 * @since 1.0
 * 
 */
public abstract class Effect extends JSTagsTagSupport {
	
	private static final long serialVersionUID = -782217420861848874L;
	
	private String shootersSelector; 
	private String event; 
	private String targetId; 
	
	private String beforeStart; 
	private String beforeSetup; 
	private String beforeUpdate; 
	private String afterUpdate; 
	private String beforeFinish; 
	private String afterFinish; 	
	
	private String transition;
	private String duration; 
		
	private String fps;
		
	private String from;
	private String to;
	
	private String delay;
	private String queue;
	
	
	/**
	 * @return the JavaScript function name to be executed right before the effect starts
	 */
	public final String getBeforeStart() {
		return beforeStart;
	}
	/**
	 * @param beforeStart the JavaScript function name to be executed right before the effect starts
	 */
	public final void setBeforeStart(String beforeStart) {
		this.beforeStart = beforeStart;
	}

	/**
	 * @return the JavaScript function name to be executed right before the effect is configured
	 */
	public final String getBeforeSetup() {
		return beforeSetup;
	}

	/**
	 * @param beforeSetup the JavaScript function name to be executed right before the effect is configured
	 */
	public final void setBeforeSetup(String beforeSetup) {
		this.beforeSetup = beforeSetup;
	}

	/**
	 * @return the JavaScript function name to be executed right at every frame, before 
	 * any change to be performed 
	 */
	public final String getBeforeUpdate() {
		return beforeUpdate;
	}

	/**
	 * @param beforeUpdate the JavaScript function name to be executed right at every frame, before 
	 * any change to be performed 
	 */
	public final void setBeforeUpdate(String beforeUpdate) {
		this.beforeUpdate = beforeUpdate;
	}
	
	/**
	 * @return the JavaScript function to be executed after the effect finishes
	 */
	public final String getAfterFinish() {
		return afterFinish;
	}
	
	/**
	 * @param afterFinish the JavaScript function to be executed after the effect finishes
	 */
	public final void setAfterFinish(String afterFinish) {
		this.afterFinish = afterFinish;
	}
	
	/**
	 * @return the JavaScript function to be executed after every update on the target object
	 */
	public final String getAfterUpdate() {
		return afterUpdate;
	}
	
	/**
	 * @param afterUpdate the JavaScript function to be executed after every update on the target object
	 */
	public final void setAfterUpdate(String afterUpdate) {
		this.afterUpdate = afterUpdate;
	}
	
	/**
	 * @return the JavaScript function to be executed before the effect finishes
	 */
	public final String getBeforeFinish() {
		return beforeFinish;
	}
	
	/**
	 * @param beforeFinish the JavaScript function to be executed before the effect execution finishes
	 */
	public final void setBeforeFinish(String beforeFinish) {
		this.beforeFinish = beforeFinish;
	}
	
	/**
	 * @return the time (in seconds) the effect will last. Default value is <code>1.0</code>
	 */
	public final String getDuration() {
		return duration;
	}

	/**
	 * @param duration the time (in seconds) the effect will last. Default value is <code>1.0</code>
	 */
	public final void setDuration(String duration) {
		this.duration = duration;
	}

	/**
	 * @return the event name on which the effect will be fired
	 */
	public final String getEvent() {
		return event;
	}

	/**
	 * @param event the event name on which the effect will be fired
	 */
	public final void setEvent(String event) {
		this.event = event;
	}

	/**
	 * @return the total number of frames to be executed per second. Default value is 
	 * <code>25.0</code>
	 */
	public final String getFps() {
		return fps;
	}

	/**
	 * @param fps the total number of frames to be executed every second. Default value is 
	 * <code>25.0</code>
	 */
	public final void setFps(String fps) {
		this.fps = fps;
	}
	
	/**
	 * @return the starting point of the transition, a float between <code>0.0</code> 
	 * and <code>1.0</code>. Default value is <code>0.0</code>
	 */
	public final String getFrom() {
		return from;
	}	

	/**
	 * @param from the starting point of the transition, a float between <code>0.0</code> 
	 * and <code>1.0</code>. Default value is <code>0.0</code>
	 */
	public final void setFrom(String from) {
		this.from = from;
	}
		
	/**
	 * @return the element name that will fire the effect when the event be fired
	 */
	public final String getShootersSelector() {
		return shootersSelector;
	}

	/**
	 * @param shootersSelector the element id that will fire the effect when the event be fired
	 */
	public final void setShootersSelector(String shooterId) {
		this.shootersSelector = shooterId;
	}

	/**
	 * @return the element id the effect will be fired on
	 */
	public final String getTargetId() {
		return targetId;
	}

	/**
	 * @param targetId the element name the effect will be fired on
	 */
	public final void setTargetId(String targetId) {
		this.targetId = targetId;
	}

	/**
	 * @return the end point of the transition, a float between <code>0.0</code> and 
	 * <code>1.0</code>. Default value is <code>1.0</code>
	 */
	public final String getTo() {
		return to;
	}

	/**
	 * @param to the end point of the transition, a float between <code>0.0</code> and 
	 * <code>1.0</code>. Default value is <code>1.0</code>
	 */
	public final void setTo(String to) {
		this.to = to;
	}

	/**
	 * @return the type of transition to be used with the effect. Default value is
	 * <code>MochiKit.Visual.Transitions.sinoidal</code>
	 * There are several available:
	 * <ul> 
	 * <li>MochiKit.Visual.Transitions.linear</li>
	 * <li>MochiKit.Visual.Transitions.sinoidal</li>
	 * <li>MochiKit.Visual.Transitions.reverse</li>
	 * <li>MochiKit.Visual.Transitions.flicker</li>
	 * <li>MochiKit.Visual.Transitions.wobble</li>
	 * <li>MochiKit.Visual.Transitions.pulse</li>
	 * <li>MochiKit.Visual.Transitions.none</li>
	 * <li>MochiKit.Visual.Transitions.full</li>
	 * </ul>
	 * Check <a href="http://mochikit.com">Mochikit</a> 
	 * for further information on the types of transitions available
	 */
	public final String getTransition() {
		return transition;
	}

	/**
	 * @param transition the type of transition to be used with the effect. Default value is
	 * <code>MochiKit.Visual.Transitions.sinoidal</code>
	 * There are several available:
	 * <ul> 
	 * <li>MochiKit.Visual.Transitions.linear</li>
	 * <li>MochiKit.Visual.Transitions.sinoidal</li>
	 * <li>MochiKit.Visual.Transitions.reverse</li>
	 * <li>MochiKit.Visual.Transitions.flicker</li>
	 * <li>MochiKit.Visual.Transitions.wobble</li>
	 * <li>MochiKit.Visual.Transitions.pulse</li>
	 * <li>MochiKit.Visual.Transitions.none</li>
	 * <li>MochiKit.Visual.Transitions.full</li>
	 * </ul>
	 * Check <a href="http://mochikit.com">Mochikit</a> 
	 * for further information on the types of transitions available
	 */
	public final void setTransition(String transition) {
		this.transition = transition;
	}

	/**
	 * @return how many seconds will pass before the effect starts executing. Default
	 * value is <code>0.0</code>
	 */
	public final String getDelay() {
		return delay;
	}
	
	/**
	 * @param delay how many seconds will pass before the effect starts executing. Default
	 * value is <code>0.0</code>
	 */
	public final void setDelay(String delay) {
		this.delay = delay;
	}
	
	/**
	 * @return the queuing options. When used with a string, can be <code>front</code> 
	 * or <code>end</code> to queue the effect in the global effects queue 
	 * at the beginning or end. It can also be a queue parameter object that 
	 * can have <code>{position: front|end, scope: elementId|global, limit: 1}</code>
	 * Check <a href="http://mochikit.com">
	 * Mochikit</a> for further information on queues
	 */
	public final String getQueue() {
		return queue;
	}

	/**
	 * @param queue the queuing options. When used with a string, can be <code>front</code> 
	 * or <code>end</code> to queue the effect in the global effects queue 
	 * at the beginning or end. It can also be a queue parameter object that 
	 * can have <code>{position: front|end, scope: elementId|global, limit: 1}</code>
	 * Check <a href="http://mochikit.com">
	 * Mochikit</a> for further information on queues
	 */
	public final void setQueue(String queue) {
		this.queue = queue;
	}
		
	/**
	 * It selects those effect properties used to configure it and returns
	 * them together with their values (that excludes: <code>shootersSelector,
	 * targetId, event</code>)
	 * 
	 * @return a {@link JSONObject} containing all the available properties
	 * for the effect and their values in the form <code>{propertyName1: propertyValue1, 
	 * propertyName2: propertyValue2, ...}</code> pairs
	 */
	protected JSONObject getOptions() {
						
		JSONObject options = new JSONObject();
		if (!StringUtils.isEmpty(duration)) {
			options.put("duration", new Float(duration));
		}
		
		if (!StringUtils.isEmpty(fps)) {
			options.put("fps", new Float(fps));
		}
		
		if (!StringUtils.isEmpty(to)) {
			options.put("to", new Float(to));			
		}		
				
		if (!StringUtils.isEmpty(from)) {
			options.put("from", new Float(from));			
		}
			
		if (!StringUtils.isEmpty(beforeStart)) {
			options.put("beforeStart", new JSONFunction(beforeStart));
		}
		if (!StringUtils.isEmpty(beforeSetup)) {
			options.put("beforeSetup", new JSONFunction(beforeSetup));
		}
		if (!StringUtils.isEmpty(beforeUpdate)) {
			options.put("beforeUpdate", new JSONFunction(beforeUpdate));
		}
		if (!StringUtils.isEmpty(afterUpdate)) {
			options.put("afterUpdate", new JSONFunction(afterUpdate));
		}
		if (!StringUtils.isEmpty(beforeFinish)) {
			options.put("beforeFinish", new JSONFunction(beforeFinish));
		}
		if (!StringUtils.isEmpty(afterFinish)) {
			options.put("afterFinish", new JSONFunction(afterFinish));			
		}
		
		if (!StringUtils.isEmpty(transition)) {
			String[] params = {"pos"};
			options.put("transition", new JSONFunction(params, "return " + transition + "(pos)"));			
		}
		
		if (!StringUtils.isEmpty(queue)) {	
			options.put("queue", queue);			
		}
				
		if (!StringUtils.isEmpty(delay)) {	
			options.put("delay", new Float(delay));			
		}		
		
		return options;
				
	}		
	
	/* (non-Javadoc)
	 * @see org.jstags.utils.tags.JSTagsTagSupport#getTemplateNameValuePairs()
	 */
	protected Map<String, String> getTemplateNameValuePairs() {
		Map<String, String> map = new HashMap<String, String>();
						
		JSONObject jsObject = new JSONObject();
		jsObject.put(TemplateConstants.TARGET_ID_PARAM_NAME, getTargetId());
		jsObject.put(TemplateConstants.SHOOTERS_SELECTOR_PARAM_NAME, getShootersSelector());
		jsObject.put(TemplateConstants.EVENT_PARAM_NAME, getEvent());
		jsObject.put(TemplateConstants.OPTIONS_PARAM_NAME, getOptions().toString(TemplateConstants.INDENT_FACTOR));
		map.put(TemplateConstants.PARAMETERS_VAR_NAME, jsObject.toString(TemplateConstants.INDENT_FACTOR));
				
		return map;
	}
}
