/**
 * Round.java $Revision: 206 $ $Date: 2007-10-12 15:04:38 +0200 (Fri, 12 Oct 2007) $
 *
 * Copyright 2007 jstags
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sourceforge.jstags.tags.effects;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.jsp.JspException;

import net.sf.json.JSONObject;
import net.sourceforge.jstags.naming.TemplateConstants;
import net.sourceforge.jstags.tags.JSTagsTagSupport;

import org.apache.commons.lang.StringUtils;

/**
 * It rounds the borders of an element(a container).
 * 
 * This effect depends on the javascript library 
 * <a href="http://mochikit.com/">Mochikit</a> 
 * 
 * @author sschz@users.sourceforge.net
 * @since 1.0
 */
public class Round extends JSTagsTagSupport {
	
	private static final long serialVersionUID = -4590488115818877746L;

	private String corners;
	
	private String color;
	
	private String bgColor;
	
	private String borderColor;
	
	private String targetsSelector;

	/**
	 * @return color(in the form #ffff22) to be used as the background of the 
	 * element top and bottom. The default value is fromParent so that,
	 * the part of the corner outside the rounded border don't be 
	 * distinguishable from the parent
	 * 
	 */
	public String getBgColor() {
		return bgColor;
	}

	/**
	 * @param bgColor color(i.e. #887733) to be used as the background of the 
	 * element top and bottom. The default value is fromParent so that,
	 * the part of the corner outside the rounded border don't be 
	 * distinguishable from the parent
	 */
	public void setBgColor(String bgColor) {
		this.bgColor = bgColor;
	}

	/**
	 * @return the color to be used for the border of the 
	 * target element(i.e. #ff0000)
	 */
	public String getBorderColor() {
		return borderColor;
	}

	/**
	 * @param border color to be used for the border of the
	 * target element(i.e. #ff0000)
	 */
	public void setBorderColor(String borderColor) {
		this.borderColor = borderColor;
	}

	/**
	 * @return color(i.e. #ffff00) to be used to fill in the top
	 * and bottom of the element. The default value is fromElement so that,
	 * the part of the top and bottom borders don't be 
	 * distinguishable from the element background
	 */
	public String getColor() {
		return color;
	}

	/**
	 * @param color {@link String} in the form #ffff00 to be used to 
	 * fill with that color the top and bottom of the element. The default value is 
	 * fromElement so that, the part of the top and bottom borders don't be 
	 * distinguishable from the element background
	 */
	public void setColor(String color) {
		this.color = color;
	}

	/**
	 * @return a String representing a CSS Selector used to retrieve
	 * the elements that will be the target of this effect, that is,
	 * the ones that will be displayed with a rounded corner
	 */
	public String getTargetsSelector() {
		return targetsSelector;
	}

	/**
	 * @param targetsSelector a {@link String} representing a CSS Selector used to retrieve
	 * the elements that will be the target of this effect, that is,
	 * the ones that will be displayed with a rounded corner
	 */
	public void setTargetsSelector(String targetsSelector) {
		this.targetsSelector = targetsSelector;
	}
	
	/**
	 * 
	 * The valid values may be a combination of:
	 * <ul>
	 * <li>all</li>
	 * <li>top</li>
	 * <li>bottom</li>
	 * <li>tl: top left</li>
	 * <li>bl: bottom left</li>
	 * <li>tr: top right</li>
	 * <li>br: bottom right</li>
	 * </ul>
	 * 
	 * @return the corners of the element the effect will be
	 * applied to
	 */
	public String getCorners() {
		return corners;
	}

	/**
	 * It sets the corners that will be displayed rounded
	 * (i.e. <code>bl tr</code>). The valid values may be a combination of:
	 * <ul>
	 * <li>all</li>
	 * <li>top</li>
	 * <li>bottom</li>
	 * <li>tl: top left</li>
	 * <li>bl: bottom left</li>
	 * <li>tr: top right</li>
	 * <li>br: bottom right</li>
	 * </ul>
	 * 
	 * @param corners the corners of the element the effect will be
	 * applied to
	 */
	public void setCorners(String corners) {
		this.corners = corners;
	}
	
	/* (non-Javadoc)
	 * @see org.jstags.utils.tags.JSTagsTagSupport#getTemplateName()
	 */
	protected String getTemplateName() throws JspException {
		return TemplateConstants.ROUND_TEMPLATE_NAME;
	}
		
	/**
	 * It selects those effect properties used to configure it and returns
	 * them together with their values (that excludes: <code>shootersSelector</code>)
	 * @return a {@link JSONObject} containing all the available properties
	 * for the effect and their values in the form <code>{propertyName1: propertyValue1, 
	 * propertyName2: propertyValue2, ...}</code> pairs
	 */
	protected JSONObject getOptions() {
						
		JSONObject options = new JSONObject();
		if (!StringUtils.isEmpty(getColor())) {
			options.put("color", getColor());
		}
		
		if (!StringUtils.isEmpty(getCorners())) {
			options.put("corners", getCorners());
		}
		
		if (!StringUtils.isEmpty(getBgColor())) {
			options.put("bgColor", getBgColor());
		}
		
		options.put("border", borderColor);			
								
		return options;
				
	}		
	
	/* (non-Javadoc)
	 * @see org.jstags.utils.tags.JSTagsTagSupport#getTemplateParamValuePairs()
	 */
	protected Map<String, String> getTemplateNameValuePairs() {
		Map<String, String> map = new HashMap<String, String>();
		
		map.put(TemplateConstants.TARGETS_SELECTOR_VAR_NAME, getTargetsSelector());
		map.put(TemplateConstants.OPTIONS_VAR_NAME, getOptions().toString(TemplateConstants.INDENT_FACTOR));
				
		return map;
	}

}
