/**
 * TableValidator.java $Revision: 252 $ $Date: 2007-10-13 15:33:12 +0200 (Sat, 13 Oct 2007) $
 *
 * Copyright ${year} jstags
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sourceforge.jstags.tags.table;

import java.util.ArrayList;
import java.util.Collection;

import javax.servlet.jsp.tagext.TagData;
import javax.servlet.jsp.tagext.TagExtraInfo;
import javax.servlet.jsp.tagext.ValidationMessage;

import net.sf.json.util.JSONUtils;
import net.sourceforge.jstags.naming.TableConstants;
import net.sourceforge.jstags.utils.validation.NumberUtils;
import net.sourceforge.jstags.utils.validation.TagsUtils;

import org.apache.log4j.Logger;

/**
 * Checks the validity of the params provided:
 * <ul>
 * <li>targetId</li>
 * <li>tableClass</li> 
 * <li>oddRowClass</li>
 * 
 * <li>url</li>
 * <li>frequency</li>
 * <li>decay</li>
 * 
 * <li>event</li>
 * <li>shooterId</li>
 * 
 * <li>formNames</li>
 * <li>params</li>
 * <li>nameValuePairs</li>
 * 
 * <li>startIndex</li>
 * <li>count</li>
 * 
 * <li>function</li>
 * </ul>
 *
 * @author sschz@users.sourceforge.net
 * @since 1.0
 * 
 */
public class TableValidator extends TagExtraInfo {
	/**
	 * Logger for this class
	 */
	private static final Logger logger = Logger
			.getLogger(TableValidator.class);


	/* (non-Javadoc)
	 * @see javax.servlet.jsp.tagext.TagExtraInfo#validate(javax.servlet.jsp.tagext.TagData)
	 */
	public ValidationMessage[] validate(TagData data) {

		Collection<ValidationMessage> messages = new ArrayList<ValidationMessage>();

		Object frequency = data.getAttribute(TableConstants.FREQUENCY_TAG_ATTR_NAME);
		Object decay = data.getAttribute(TableConstants.DECAY_TAG_ATTR_NAME);
		Object nameValuePairs = data.getAttribute(TableConstants.NAME_VALUE_PAIRS_TAG_ATTR_NAME);

		Object shooterId = data.getAttribute(TableConstants.SHOOTER_ID_TAG_ATTR_NAME);
		Object event = data.getAttribute(TableConstants.EVENT_TAG_ATTR_NAME);
		Object formNames = data.getAttribute(TableConstants.FORM_NAMES_TAG_ATTR_NAME);
		Object params = data.getAttribute(TableConstants.PARAMS_TAG_ATTR_NAME);
		
		Object startIndex = data.getAttribute(TableConstants.START_INDEX_TAG_ATTR_NAME);
		Object count = data.getAttribute(TableConstants.COUNT_TAG_ATTR_NAME);
		
		// Frequency must be an integer
		if (TagsUtils.isValueAvailableAtTranslationTime(frequency)) {
			if (!NumberUtils.isInteger((String)frequency)) {
				logger.error("Param 'frequency: " + (String)frequency + "' must be an integer");
				messages.add(new ValidationMessage(data.getId(), 
						"Param 'frequency: " + (String)frequency + "' must be an integer"));
			}			
		}
		
		// Decay must be an integer
		if (TagsUtils.isValueAvailableAtTranslationTime(decay)) {
			if (!NumberUtils.isInteger((String)decay)) {
				logger.error("Param 'decay:" + (String)decay + "' must be an integer");
				messages.add(new ValidationMessage(data.getId(), 
						"Param 'decay:" + (String)decay + "' must be an integer"));
			}			
		}
			
		// nameValuePairs must be a javascript object 
		if (TagsUtils.isValueAvailableAtTranslationTime(nameValuePairs)) {
			if ((!JSONUtils.mayBeJSON((String)nameValuePairs))
					|| (JSONUtils.mayBeJSON((String)nameValuePairs) && !((String)nameValuePairs).startsWith("{"))){
				logger.error("Param 'nameValuePairs: " + (String)nameValuePairs 
						+ "' must be a String representing a valid JSON Object");
				messages.add(new ValidationMessage(data.getId(), 
						"Param 'nameValuePairs: " + (String)nameValuePairs 
						+ "' must be a String representing a valid JSON Object"));
			}
		}
			
		// formNames must be a javascript array 
		if (TagsUtils.isValueAvailableAtTranslationTime(formNames)) {
			if ((!JSONUtils.mayBeJSON((String)formNames))
					|| (JSONUtils.mayBeJSON((String)formNames) && !((String)formNames).startsWith("["))){
				logger.error("Param 'forms: " + (String)formNames + "' must be a String representing a valid JSON Array");
				messages.add(new ValidationMessage(data.getId(), 
						"Param 'forms: " + (String)formNames + "' must be a String representing a valid JSON Array"));
			}
		}
		
		// params must be a javascript array
		if (TagsUtils.isValueAvailableAtTranslationTime(params)) {
			if ((!JSONUtils.mayBeJSON((String)params))
					|| (JSONUtils.mayBeJSON((String)params) && !((String)params).startsWith("["))){
				logger.error("Param 'params: " + (String)params + "' must be a String representing a valid JSON Array");
				messages.add(new ValidationMessage(data.getId(), 
						"Param 'params: " + (String)params + "' must be a String representing a valid JSON Array"));
			}
		}
		
		// StartIndex must be an integer
		if (TagsUtils.isValueAvailableAtTranslationTime(startIndex)) {
			if (!NumberUtils.isInteger((String)startIndex)) {
				logger.error("Param 'startIndex:" + (String)startIndex + "' must be an integer");
				messages.add(new ValidationMessage(data.getId(), 
						"Param 'startIndex:" + (String)startIndex + "' must be an integer"));
			}			
		}
		
		// Count must be an integer
		if (TagsUtils.isValueAvailableAtTranslationTime(count)) {
			if (!NumberUtils.isInteger((String)count)) {
				logger.error("Param 'count:" + (String)count + "' must be an integer");
				messages.add(new ValidationMessage(data.getId(), 
						"Param 'count:" + (String)count + "' is not an integer"));
			}			
		}
		
		if ((count != null)	|| (startIndex != null)) {
			if ((count == null) || (startIndex == null)) {
				logger.error("Params 'count' and 'startIndex' must be set either both or none");
				messages.add(new ValidationMessage(data.getId(), 
						"Params 'count' and 'startIndex' must be set either both or none"));
			}
		}
		
		if ((shooterId != null) || (event != null)) {
			if ((shooterId == null) || (event == null)) {
				logger.error("Params 'shooterId' and 'event' must be set either both or none");
				messages.add(new ValidationMessage(data.getId(), 
						"Params 'shooterId' and 'event' must be set either both or none"));
			}
		}
		
		return (ValidationMessage[])messages.toArray(new ValidationMessage[messages.size()]);		
	}	
	
}
