/**
 * XSLTTransformer.java $Revision: 417 $ $Date: 2007-10-31 00:50:11 +0100 (Wed, 31 Oct 2007) $
 *
 * Copyright ${year} jstags
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sourceforge.jstags.tags.transform;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.jsp.JspException;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import net.sourceforge.jstags.naming.TemplateConstants;
import net.sourceforge.jstags.tags.JSTagsTagSupport;

import org.apache.commons.lang.StringUtils;

/**
 * It allows the parsing of a XML document by using an XSL. The
 * parsed result is inserted in the page. Both, the XML and XSL 
 * documents, are loaded by means of XMLHttpRequest  
 * 
 * This tag depends on the JavaScript library 
 * <a href="http://mochikit.com">Mochikit</a> together with
 * <a href="http://jstags.sourceforge.net/">JSTags</a> 
 * JavaScript libraries
 * 
 * @author sschz@users.sourceforge.net
 * @since 1.0
 * 
 */
public class XSLTTransformer extends JSTagsTagSupport {
	
	private static final long serialVersionUID = 2795300707076655393L;
	
	private String shootersSelector; 
	private String event; 
	private String targetId; 
	
	private String xmlUrl;
	private String xslUrl;
	
	private String formNames;
	private String params;
	private String nameValuePairs;

	/**
	 * @return <code>,</code> separated array of values in the form ['name1', 'name2'], 
	 * all the params of each form named in the attribute value (identified by the 
	 * name provided here) will be sent with every request of the XML data
	 */
	public String getFormNames() {
		return formNames;
	}
	
	/**
	 * @param formNames <code>,</code> separated array of values in the form ['name1', 'name2'], 
	 * all the params of each form named in the attribute value (identified by the 
	 * name provided here) will be sent with every request of the XML data
	 */
	public void setFormNames(String formNames) {
		this.formNames = formNames;
	}
	
	/**
	 * These params will be added, with the value given here, to the 
	 * request of the XML data
	 * 
	 * @return a <code>,</code> separated object of pairs <code>name:value</code>.
	 * i.e. <code>{name1:'value1',name2:'value2'}</code>
	 * 
	 */
	public String getNameValuePairs() {
		return nameValuePairs;
	}
	
	/**
	 * These params will be added, with the value given here, to the 
	 * request of the XML data
	 * 
	 * @param nameValuePairs a <code>,</code> separated object of pairs <code>name:value</code>.
	 * i.e. <code>{name1:'value1',name2:'value2'}</code>
	 * 
	 */
	public void setNameValuePairs(String nameValuePairs) {
		this.nameValuePairs = nameValuePairs;
	}
	
	/**
	 * @return <code>,</code> separated array of values in the form ['param1', 'param2'], 
	 * the value of every param included in its value (identified by the its id provided 
	 * here) will be sent with every request of the XML data.   
	 */
	public String getParams() {
		return params;
	}
	
	/**
	 * @param params <code>,</code> separated array of values in the form ['param1', 'param2'], 
	 * the value of every param included in its value (identified by the its id provided 
	 * here) will be sent with every request of the XML data.   
	 */
	public void setParams(String params) {
		this.params = params;
	}
	
	/**
	 * @return the event name on which the effect will be fired
	 */
	public String getEvent() {
		return event;
	}
	
	/**
	 * @param event the event name on which the effect will be fired
	 */ 
	public void setEvent(String event) {
		this.event = event;
	}
	
	/**
	 * @return the element name that will fire the effect when the event be fired
	 */
	public String getShootersSelector() {
		return shootersSelector;
	}
	
	/**
	 * @param shootersSelector the element name that will fire the effect when the event be fired
	 */
	public void setShootersSelector(String shootersSelector) {
		this.shootersSelector = shootersSelector;
	}
	
	/**
	 * @return the element id the effect will be fired on
	 */
	public String getTargetId() {
		return targetId;
	}
	
	/**
	 * @param targetId the element id the effect will be fired on
	 */
	public void setTargetId(String targetId) {
		this.targetId = targetId;
	}
	
	/**
	 * @return url where the xml data will be retrieved from
	 */
	public String getXmlUrl() {
		return xmlUrl;
	}
	/**
	 * @param xmlUrl url where the xml data will be retrieved from
	 */
	public void setXmlUrl(String xmlUrl) {
		this.xmlUrl = xmlUrl;
	}
	/**
	 * @return url where the xsl will be retrieved from
	 */
	public String getXslUrl() {
		return xslUrl;
	}
	/**
	 * @param xslUrl url where the xsl will be retrieved from
	 */
	public void setXslUrl(String xslUrl) {
		this.xslUrl = xslUrl;
	}
	
	
	/* (non-Javadoc)
	 * @see org.jstags.utils.tags.JSTagsTagSupport#getTemplateName()
	 */
	@Override
	protected String getTemplateName() throws JspException {
		return TemplateConstants.XSLT_TRANSFORMATION_TEMPLATE_NAME;
	}
	
	/* (non-Javadoc)
	 * @see org.jstags.utils.tags.JSTagsTagSupport#getTemplateNameValuePairs()
	 */
	@Override
	protected Map<String, String> getTemplateNameValuePairs() {
		Map<String, String> map = new HashMap<String, String>();
		
		JSONObject jsObject = new JSONObject();
		
		jsObject.put(TemplateConstants.TARGET_ID_PARAM_NAME, getTargetId());
		if (!StringUtils.isEmpty(getShootersSelector())) {
			jsObject.put(TemplateConstants.SHOOTERS_SELECTOR_PARAM_NAME, getShootersSelector());
			jsObject.put(TemplateConstants.EVENT_PARAM_NAME, getEvent());			
		} 		
		
		jsObject.put(TemplateConstants.XML_URL_PARAM_NAME, getXmlUrl());
		jsObject.put(TemplateConstants.XSL_URL_PARAM_NAME, getXslUrl());
				
		// Parameters used to get the XML data
		JSONObject jsonObject = new JSONObject();
		if (!StringUtils.isEmpty(getNameValuePairs())) {
			jsonObject.put(TemplateConstants.NAME_VALUE_PAIRS_PARAM_NAME, JSONObject.fromString(getNameValuePairs()));
		}
		if (!StringUtils.isEmpty(getParams())) {
			jsonObject.put(TemplateConstants.FORMS_PARAMS_PARAM_NAME, JSONObject.fromString(getParams()));
		} 
		if (!StringUtils.isEmpty(getFormNames())) {
			jsonObject.put(TemplateConstants.FORM_NAMES_PARAM_NAME, JSONArray.fromString(getFormNames()));
		}		
		jsObject.put(TemplateConstants.XML_URL_PARAMETERS_PARAM_NAME, jsonObject.toString(TemplateConstants.INDENT_FACTOR));
		
		map.put(TemplateConstants.PARAMETERS_VAR_NAME, jsObject.toString(TemplateConstants.INDENT_FACTOR));
		
		return map;
	}

}
