/**
 * TemplateUtils.java $Revision: 207 $ $Date: 2007-10-12 15:08:01 +0200 (Fri, 12 Oct 2007) $
 *
 * Copyright 2007 jstags
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sourceforge.jstags.utils.template;

import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.Map;
import java.util.Map.Entry;

import net.sourceforge.jstags.exceptions.TemplateException;
import net.sourceforge.jstags.naming.TemplateConstants;

import org.apache.log4j.Logger;


/**
 * <p>This class contains a set of methods to work with templates.</p>
 * <p>Every tag included in the library will generate a chunk of JavaScript code. Such code
 * is generated by means of templates.</p>
 * <p>A template is just a text file that contains a collection of <code>%VAR_NAME%</code>
 * elements representing variables to be replaced with the data used in every tag 
 * instance. Every time a tag is created, the JavaScript piece of code neeeded will be
 * created. First of all, the template associated with the tag created will be retrieve.
 * After that, the values of every <code>%VAR_NAME%</code> in that template will
 * be replaced with the ones set in the tag. The code generated will contain the 
 * necessary logic to insert in the page</p>
 * 
 * @author sschz@users.sourceforge.net
 * @since 1.0
 * 
 */
public class TemplateUtils {
	/**
	 * Logger for this class
	 */
	private static final Logger logger = Logger.getLogger(TemplateUtils.class);
			
	/**
	 * Locates <code>templateFileName</code> in {@link TemplateConstants#TEMPLATES_FOLDER}
	 * and returns its content
	 * @param templateFileName name of the template to be retrieved
	 * @return a {@link String} with the templateFileName contents
	 * @throws TemplateException 
	 */
	public static String getTemplateAsString(String templateFileName) 
		throws TemplateException {

		try {
			InputStream inputStream = TemplateUtils.class
										.getResourceAsStream(TemplateConstants.TEMPLATES_FOLDER 
												+ "/" + templateFileName);
			
			ByteArrayOutputStream fileAsByteArrayOS = new ByteArrayOutputStream();
			
			long length = 1024;
			
			byte[] line = new byte[(int)length];			
			int read = 0;
			while ((read = inputStream.read(line, 0, 1024)) >= 0) {
				fileAsByteArrayOS.write(line, 0, read);				
			}
			
			String output = new String(fileAsByteArrayOS.toByteArray());
			
			logger.debug("Template retrieved: " + output);
						
			return output;
						
		} catch (FileNotFoundException e) {
			logger.error("The file " + templateFileName 
					+ " located in " + TemplateConstants.TEMPLATES_FOLDER
					+ " has not been found", e);
			throw new TemplateException("The file " + templateFileName 
					+ " located in " + TemplateConstants.TEMPLATES_FOLDER
					+ " has not been found", e);			
		} catch (Exception e) {
			logger.error("There has been an exception trying to access file " 
					+ templateFileName + " located in " 
					+ TemplateConstants.TEMPLATES_FOLDER, e);
			throw new TemplateException("There has been an exception trying to access file " 
					+ templateFileName + " located in " 
					+ TemplateConstants.TEMPLATES_FOLDER, e);			
		} 
		
	}
	
	/**
	 * Given a template content, it replaces the variables in it returning the new content
	 * @param template content to be used as a template
	 * @param nameValuePairs {@link Map} of ({@link String}, {@link String}) pairs representing
	 * the data to be replaced in the template. Every {@link Entry} represents a variable name
	 * and its value 
	 * @return
	 */
	public static String replaceValuesInTemplate(String template, 
			Map<String, String> nameValuePairs) {

		for (String varName : nameValuePairs.keySet()) {
			template = template.replaceAll(varName, nameValuePairs.get(varName));
		}
		logger.debug("Template after replacing the variables: " + template);
		return template;
	}
	
}
